@@ -1,6 +1,7 @@
 #!/usr/bin/python3
 
 # Copyright 2020 Google LLC
+# Copyright 2021 Fraunhofer FKIE
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,18 +16,17 @@
 # limitations under the License.
 """An example of fuzzing in Python."""
 
-import struct
+import atheris
 import sys
 
-import atheris
-
-
-def CodeBeingFuzzed(number):
-  """Raises an exception if number is 17."""
-  if number == 17:
-    raise RuntimeError('Number was seventeen!')
+# This tells Atheris to instrument all functions in the `struct` and
+# `example_library` modules.
+with atheris.instrument_imports():
+  import struct
+  import example_library
 
 
+@atheris.instrument_func  # Instrument the TestOneInput function itself
 def TestOneInput(data):
   """The entry point for our fuzzer.
 
@@ -42,8 +42,7 @@ def TestOneInput(data):
     return  # Input must be 4 byte integer.
 
   number, = struct.unpack('<I', data)
-  CodeBeingFuzzed(number)
-
+  example_library.CodeBeingFuzzed(number)
 
 atheris.Setup(sys.argv, TestOneInput)
 atheris.Fuzz()
